@@ -53,6 +53,7 @@ in this Software without prior written authorization from the X Consortium.
 #endif
 #ifdef DPMSExtension
 #include <X11/extensions/dpmsconst.h>
+#include "dpmsproc.h"
 #endif
 #include "protocol-versions.h"
 
@@ -426,7 +427,7 @@ SendScreenSaverNotify(ScreenPtr pScreen, int state, Bool forced)
     }
 }
 
-static void
+static void _X_COLD
 SScreenSaverNotifyEvent(xScreenSaverNotifyEvent * from,
                         xScreenSaverNotifyEvent * to)
 {
@@ -1185,6 +1186,8 @@ ProcScreenSaverUnsetAttributes(ClientPtr client)
         PanoramiXRes *draw;
         int rc, i;
 
+        REQUEST_SIZE_MATCH(xScreenSaverUnsetAttributesReq);
+
         rc = dixLookupResourceByClass((void **) &draw, stuff->drawable,
                                       XRC_DRAWABLE, client, DixWriteAccess);
         if (rc != Success)
@@ -1265,19 +1268,17 @@ ProcScreenSaverQueryVersion,
         ProcScreenSaverSetAttributes,
         ProcScreenSaverUnsetAttributes, ProcScreenSaverSuspend,};
 
-#define NUM_REQUESTS	((sizeof NormalVector) / (sizeof NormalVector[0]))
-
 static int
 ProcScreenSaverDispatch(ClientPtr client)
 {
     REQUEST(xReq);
 
-    if (stuff->data < NUM_REQUESTS)
+    if (stuff->data < ARRAY_SIZE(NormalVector))
         return (*NormalVector[stuff->data]) (client);
     return BadRequest;
 }
 
-static int
+static int _X_COLD
 SProcScreenSaverQueryVersion(ClientPtr client)
 {
     REQUEST(xScreenSaverQueryVersionReq);
@@ -1286,7 +1287,7 @@ SProcScreenSaverQueryVersion(ClientPtr client)
     return ProcScreenSaverQueryVersion(client);
 }
 
-static int
+static int _X_COLD
 SProcScreenSaverQueryInfo(ClientPtr client)
 {
     REQUEST(xScreenSaverQueryInfoReq);
@@ -1296,7 +1297,7 @@ SProcScreenSaverQueryInfo(ClientPtr client)
     return ProcScreenSaverQueryInfo(client);
 }
 
-static int
+static int _X_COLD
 SProcScreenSaverSelectInput(ClientPtr client)
 {
     REQUEST(xScreenSaverSelectInputReq);
@@ -1307,7 +1308,7 @@ SProcScreenSaverSelectInput(ClientPtr client)
     return ProcScreenSaverSelectInput(client);
 }
 
-static int
+static int _X_COLD
 SProcScreenSaverSetAttributes(ClientPtr client)
 {
     REQUEST(xScreenSaverSetAttributesReq);
@@ -1325,7 +1326,7 @@ SProcScreenSaverSetAttributes(ClientPtr client)
     return ProcScreenSaverSetAttributes(client);
 }
 
-static int
+static int _X_COLD
 SProcScreenSaverUnsetAttributes(ClientPtr client)
 {
     REQUEST(xScreenSaverUnsetAttributesReq);
@@ -1335,7 +1336,7 @@ SProcScreenSaverUnsetAttributes(ClientPtr client)
     return ProcScreenSaverUnsetAttributes(client);
 }
 
-static int
+static int _X_COLD
 SProcScreenSaverSuspend(ClientPtr client)
 {
     REQUEST(xScreenSaverSuspendReq);
@@ -1352,12 +1353,12 @@ SProcScreenSaverQueryVersion,
         SProcScreenSaverSetAttributes,
         SProcScreenSaverUnsetAttributes, SProcScreenSaverSuspend,};
 
-static int
+static int _X_COLD
 SProcScreenSaverDispatch(ClientPtr client)
 {
     REQUEST(xReq);
 
-    if (stuff->data < NUM_REQUESTS)
+    if (stuff->data < ARRAY_SIZE(NormalVector))
         return (*SwappedVector[stuff->data]) (client);
     return BadRequest;
 }
