@@ -89,7 +89,7 @@ proc_present_pixmap(ClientPtr client)
     ret = dixLookupWindow(&window, stuff->window, client, DixWriteAccess);
     if (ret != Success)
         return ret;
-    ret = dixLookupResourceByType((pointer *) &pixmap, stuff->pixmap, RT_PIXMAP, client, DixReadAccess);
+    ret = dixLookupResourceByType((void **) &pixmap, stuff->pixmap, RT_PIXMAP, client, DixReadAccess);
     if (ret != Success)
         return ret;
 
@@ -184,8 +184,6 @@ proc_present_select_input (ClientPtr client)
 
     REQUEST_SIZE_MATCH(xPresentSelectInputReq);
 
-    LEGAL_NEW_RESOURCE(stuff->eid, client);
-
     rc = dixLookupWindow(&window, stuff->window, client, DixGetAttrAccess);
     if (rc != Success)
         return rc;
@@ -210,6 +208,7 @@ proc_present_query_capabilities (ClientPtr client)
     RRCrtcPtr   crtc = NULL;
     int         r;
 
+    REQUEST_SIZE_MATCH(xPresentQueryCapabilitiesReq);
     r = dixLookupWindow(&window, stuff->target, client, DixGetAttrAccess);
     switch (r) {
     case Success:
@@ -233,7 +232,7 @@ proc_present_query_capabilities (ClientPtr client)
     return Success;
 }
 
-int (*proc_present_vector[PresentNumberRequests]) (ClientPtr) = {
+static int (*proc_present_vector[PresentNumberRequests]) (ClientPtr) = {
     proc_present_query_version,            /* 0 */
     proc_present_pixmap,                   /* 1 */
     proc_present_notify_msc,               /* 2 */
@@ -250,10 +249,11 @@ proc_present_dispatch(ClientPtr client)
     return (*proc_present_vector[stuff->data]) (client);
 }
 
-static int
+static int _X_COLD
 sproc_present_query_version(ClientPtr client)
 {
     REQUEST(xPresentQueryVersionReq);
+    REQUEST_SIZE_MATCH(xPresentQueryVersionReq);
 
     swaps(&stuff->length);
     swapl(&stuff->majorVersion);
@@ -261,10 +261,11 @@ sproc_present_query_version(ClientPtr client)
     return (*proc_present_vector[stuff->presentReqType]) (client);
 }
 
-static int
+static int _X_COLD
 sproc_present_pixmap(ClientPtr client)
 {
     REQUEST(xPresentPixmapReq);
+    REQUEST_AT_LEAST_SIZE(xPresentPixmapReq);
 
     swaps(&stuff->length);
     swapl(&stuff->window);
@@ -280,10 +281,11 @@ sproc_present_pixmap(ClientPtr client)
     return (*proc_present_vector[stuff->presentReqType]) (client);
 }
 
-static int
+static int _X_COLD
 sproc_present_notify_msc(ClientPtr client)
 {
     REQUEST(xPresentNotifyMSCReq);
+    REQUEST_SIZE_MATCH(xPresentNotifyMSCReq);
 
     swaps(&stuff->length);
     swapl(&stuff->window);
@@ -293,10 +295,11 @@ sproc_present_notify_msc(ClientPtr client)
     return (*proc_present_vector[stuff->presentReqType]) (client);
 }
 
-static int
+static int _X_COLD
 sproc_present_select_input (ClientPtr client)
 {
     REQUEST(xPresentSelectInputReq);
+    REQUEST_SIZE_MATCH(xPresentSelectInputReq);
 
     swaps(&stuff->length);
     swapl(&stuff->window);
@@ -304,16 +307,17 @@ sproc_present_select_input (ClientPtr client)
     return (*proc_present_vector[stuff->presentReqType]) (client);
 }
 
-static int
+static int _X_COLD
 sproc_present_query_capabilities (ClientPtr client)
 {
     REQUEST(xPresentQueryCapabilitiesReq);
+    REQUEST_SIZE_MATCH(xPresentQueryCapabilitiesReq);
     swaps(&stuff->length);
     swapl(&stuff->target);
     return (*proc_present_vector[stuff->presentReqType]) (client);
 }
 
-int (*sproc_present_vector[PresentNumberRequests]) (ClientPtr) = {
+static int (*sproc_present_vector[PresentNumberRequests]) (ClientPtr) = {
     sproc_present_query_version,           /* 0 */
     sproc_present_pixmap,                  /* 1 */
     sproc_present_notify_msc,              /* 2 */
@@ -321,7 +325,7 @@ int (*sproc_present_vector[PresentNumberRequests]) (ClientPtr) = {
     sproc_present_query_capabilities,      /* 4 */
 };
 
-int
+int _X_COLD
 sproc_present_dispatch(ClientPtr client)
 {
     REQUEST(xReq);
